# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: specio/serialization/protobuf/common.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'specio/serialization/protobuf/common.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*specio/serialization/protobuf/common.proto\x12\x0cspecio.proto\"9\n\rSpectralShape\x12\r\n\x05start\x18\x01 \x01(\x01\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x01\x12\x0c\n\x04step\x18\x03 \x01(\x01\"\x82\x01\n\x14SpectralDistribution\x12*\n\x05shape\x18\x01 \x01(\x0b\x32\x1b.specio.proto.SpectralShape\x12\x0e\n\x06values\x18\x02 \x03(\x01\x12\x12\n\nvalues_old\x18\x04 \x03(\x02\x12\x11\n\x04name\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x07\n\x05_name\",\n\tXYZ_value\x12\t\n\x01X\x18\x01 \x01(\x01\x12\t\n\x01Y\x18\x02 \x01(\x01\x12\t\n\x01Z\x18\x03 \x01(\x01\" \n\x08xy_value\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\"%\n\tcct_value\x12\x0b\n\x03\x63\x63t\x18\x01 \x01(\x01\x12\x0b\n\x03\x64uv\x18\x02 \x01(\x01\"\x1c\n\tTimestamp\x12\x0f\n\x07timestr\x18\x01 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'specio.serialization.protobuf.common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SPECTRALSHAPE']._serialized_start=60
  _globals['_SPECTRALSHAPE']._serialized_end=117
  _globals['_SPECTRALDISTRIBUTION']._serialized_start=120
  _globals['_SPECTRALDISTRIBUTION']._serialized_end=250
  _globals['_XYZ_VALUE']._serialized_start=252
  _globals['_XYZ_VALUE']._serialized_end=296
  _globals['_XY_VALUE']._serialized_start=298
  _globals['_XY_VALUE']._serialized_end=330
  _globals['_CCT_VALUE']._serialized_start=332
  _globals['_CCT_VALUE']._serialized_end=369
  _globals['_TIMESTAMP']._serialized_start=371
  _globals['_TIMESTAMP']._serialized_end=399
# @@protoc_insertion_point(module_scope)
